<?php
/**
 * Plugin Name: Vizou Quickloop
 * Plugin URI: https://explicable.ca
 * Description: A simple shortcode to create custom post loops inside pages.
 * Version: 1.0.5
 * Requires at least: 6.0
 * Tested up to: 6.7.2
 * Requires PHP: 8.0
 * Author: Dana Whittle
 * Author URI: https://vizou.com
 * License: GPLv2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: vizou-quickloop
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

function vizou_custom_quickloop($atts) {
    // Parse attributes
    $atts = shortcode_atts(array(
        'posts_per_page' => '',
        'p' => '',
        'tag' => '',
        'orderby' => 'menu_order',
        'order' => 'asc',
        'offset' => '',
        'cat' => '',
        'class' => ''
    ), $atts);

    // Build query args
    $query_args = array();
    
    if ($atts['posts_per_page'] !== '') {
        $query_args['posts_per_page'] = intval($atts['posts_per_page']);
    }
    if (!empty($atts['cat'])) {
        $query_args['cat'] = absint($atts['cat']);
    }
    if (!empty($atts['p'])) {
        $query_args['p'] = absint($atts['p']);
    }
    if (!empty($atts['orderby'])) {
        $query_args['orderby'] = sanitize_key($atts['orderby']);
    }
    if (!empty($atts['order'])) {
        $query_args['order'] = sanitize_key($atts['order']);
    }
    if (!empty($atts['tag'])) {
        $query_args['tag'] = sanitize_text_field($atts['tag']);
    }
    
    // Handle offset
    if ($atts['offset'] !== '') {
        if ($atts['posts_per_page'] === '-1') {
            $count_query = new WP_Query(array_merge($query_args, array('posts_per_page' => -1, 'fields' => 'ids')));
            $total_posts = $count_query->post_count;
            wp_reset_postdata();
            
            $query_args['posts_per_page'] = $total_posts - intval($atts['offset']);
            $query_args['offset'] = intval($atts['offset']);
        } else {
            $query_args['offset'] = intval($atts['offset']);
        }
    }

    // Start output buffering
    ob_start();
    
    // Run the query with a unique name to prevent conflicts
    $vizou_loop = new WP_Query($query_args);
    
    if ($vizou_loop->have_posts()) :
        // Check for template override
        $template = trailingslashit(get_template_directory()) . 'vizou-quickloop-template.php';
        if (file_exists($template) && is_readable($template)) {
            // Create a local scope for the template to prevent variable conflicts
            (function() use ($vizou_loop, $atts, $template) {
                include($template);
            })();
        } else {
        ?>
            <!-- START LAYOUT -->
            <div class="posts vizou-quickloop<?php echo !empty($atts['class']) ? ' ' . esc_attr($atts['class']) : ''; ?>">
                <?php while ($vizou_loop->have_posts()) : $vizou_loop->the_post(); ?>
                    <article id="post-<?php the_ID(); ?>" class="post">
                        <a href="<?php echo esc_url(get_permalink()); ?>" title="<?php echo esc_attr(get_the_title()); ?>">
                            <?php the_post_thumbnail('large'); ?>
                        </a>
                        <span class="post-content">
                            <h3>
                                <a href="<?php echo esc_url(get_permalink()); ?>" title="<?php echo esc_attr(get_the_title()); ?>"><?php echo esc_html(get_the_title()); ?></a>
                            </h3>
                            <?php echo esc_html(get_the_excerpt()); ?>
                        </span>
                        <a href="<?php echo esc_url(get_permalink()); ?>" class="read-more" title="Read more"></a>
                    </article>         
                <?php endwhile; ?>        
            </div>
            <!-- END LAYOUT -->
        <?php
        }
    endif;
    
    // Store the output before resetting post data
    $output = ob_get_clean();
    
    // Always reset the post data after the query
    wp_reset_postdata();
    
    return $output;
}

add_shortcode('vizou-quickloop', 'vizou_custom_quickloop');